from __future__ import annotations

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ERESSReranking(AbsTaskRetrieval):
    """ERESS: E-commerce Relevance Evaluation Suite for Reranking
    Dataset: https://huggingface.co/datasets/thebajajra/eress
    """

    metadata = TaskMetadata(
        name="ERESSReranking",
        description="""ERESS is a comprehensive e-commerce reranking dataset designed for holistic
    evaluation of reranking models. It includes diverse query intents including
    attribute-rich queries, navigational queries, gift/audience-specific queries,
    utility queries, and more.""",
        reference="https://huggingface.co/datasets/thebajajra/ERESSReranking",
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_5",
        dataset={
            "path": "thebajajra/ERESSReranking",
            "revision": "c095d55559370c1e8b0ae933ae26da111ae62966",
        },
        date=("1996-05-01", "2025-12-24"),
        domains=["Web", "E-commerce"],
        task_subtypes=["Product Reranking", "Query-Product Relevance"],
        license="apache-2.0",
        annotations_creators="LM-generated",  # LLM ensemble annotation
        dialect=[],
        sample_creation="found",  # Real-world queries
        prompt="Rerank products by relevance to the e-commerce query",
        bibtex_citation=r"""
@article{Bajaj2026RexRerankers,
  author = {Bajaj, Rahul and Garg, Anuj and Nupur, Jaya},
  journal = {Hugging Face Blog (Community Article)},
  month = jan,
  title = {{RexRerankers}: {SOTA} Rankers for Product Discovery and {AI} Assistants},
  url = {https://huggingface.co/blog/thebajajra/rexrerankers},
  urldate = {2026-01-24},
  year = {2026},
}
""",
    )



from .csfdcz_movie_review_sentiment_classification import (
    CSFDCZMovieReviewSentimentClassification,
    CSFDCZMovieReviewSentimentClassificationV2,
)
from .czech_product_review_sentiment_classification import (
    CzechProductReviewSentimentClassification,
    CzechProductReviewSentimentClassificationV2,
)
from .czech_so_me_sentiment_classification import (
    CzechSoMeSentimentClassification,
    CzechSoMeSentimentClassificationV2,
)
from .czech_subjectivity_classification import CzechSubjectivityClassification

__all__ = [
    "CSFDCZMovieReviewSentimentClassification",
    "CSFDCZMovieReviewSentimentClassificationV2",
    "CzechProductReviewSentimentClassification",
    "CzechProductReviewSentimentClassificationV2",
    "CzechSoMeSentimentClassification",
    "CzechSoMeSentimentClassificationV2",
    "CzechSubjectivityClassification",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class WinoGrande(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="WinoGrande",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on winogrande.",
        reference="https://winogrande.allenai.org/",
        dataset={
            "path": "mteb/WinoGrande",
            "revision": "4dec9c5666e9f84702ac614363db6d96a68bc6de",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{sakaguchi2021winogrande,
  author = {Sakaguchi, Keisuke and Bras, Ronan Le and Bhagavatula, Chandra and Choi, Yejin},
  journal = {Communications of the ACM},
  number = {9},
  pages = {99--106},
  publisher = {ACM New York, NY, USA},
  title = {Winogrande: An adversarial winograd schema challenge at scale},
  volume = {64},
  year = {2021},
}

@article{xiao2024rar,
  author = {Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2404.06347},
  title = {RAR-b: Reasoning as Retrieval Benchmark},
  year = {2024},
}
""",
        prompt={
            "query": "Given the following sentence, retrieve an appropriate answer to fill in the missing underscored part."
        },
    )



"""Models for GeoGPT-Research-Project"""

import torch

from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta

geoembedding = ModelMeta(
    name="GeoGPT-Research-Project/GeoEmbedding",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="29803c28ea7ef6871194a8ebc85ad7bfe174928e",
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        apply_instruction_to_passages=False,
        model_kwargs={"torch_dtype": torch.bfloat16},
        trust_remote_code=True,
    ),
    release_date="2025-04-22",
    n_parameters=7110660096,
    n_embedding_parameters=131_072_000,
    memory_usage_mb=27625,
    embed_dim=4096,
    license="apache-2.0",
    max_tokens=32768,
    reference="https://huggingface.co/GeoGPT-Research-Project/GeoEmbedding",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "ArguAna",
        "FEVER",
        "MSMARCO",
        "FiQA2018",
        "HotpotQA",
        "NFCorpus",
        "SciFact",
        "AmazonCounterfactualClassification",
        "AmazonPolarityClassification",
        "AmazonReviewsClassification",
        "Banking77Classification",
        "EmotionClassification",
        "MassiveIntentClassification",
        "MTOPDomainClassification",
        "MTOPIntentClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "ArxivClusteringS2S",
        "ArxivClusteringP2P",
        "MedrixvClusteringS2S",
        "MedrixvClusteringP2P",
        "BiorxivClusteringS2S",
        "BiorxivClusteringP2P",
        "TwentyNewsgroupsClustering",
        "STS12",
        "STS22",
        "STSBenchmark",
        "StackOverflowDupQuestions",
    },
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SweFaqRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SweFaqRetrieval",
        dataset={
            "path": "mteb/SweFaqRetrieval",
            "revision": "208cb812631068a4bd8b93c8b9291370b47f282c",
        },
        description="A Swedish QA dataset derived from FAQ",
        reference="https://spraakbanken.gu.se/en/resources/superlim",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="ndcg_at_10",
        date=("2000-01-01", "2024-12-31"),  # best guess
        task_subtypes=["Question answering"],
        domains=["Government", "Non-fiction", "Written"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{berdivcevskis2023superlim,
  author = {Berdi{\v{c}}evskis, Aleksandrs and Bouma, Gerlof and Kurtz, Robin and Morger, Felix and {\"O}hman, Joey and Adesam, Yvonne and Borin, Lars and Dann{\'e}lls, Dana and Forsberg, Markus and Isbister, Tim and others},
  booktitle = {Proceedings of the 2023 Conference on Empirical Methods in Natural Language Processing},
  pages = {8137--8153},
  title = {Superlim: A Swedish language understanding evaluation benchmark},
  year = {2023},
}
""",  # for the benchmark in which this dataset is used
        prompt={"query": "Retrieve answers given questions in Swedish"},
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import WhisperModel, WhisperProcessor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class WhisperAudioWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        self.model = WhisperModel.from_pretrained(model_name, revision=revision).to(
            device
        )
        self.model.eval()
        self.processor = WhisperProcessor.from_pretrained(model_name, revision=revision)
        self.sampling_rate = self.processor.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        hidden_layer: float = 1.0,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            feature_inputs = self.processor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="max_length",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.model.encoder(
                    feature_inputs.input_features,
                    output_hidden_states=True,
                )

                hidden_states = outputs.hidden_states
                no_hidden_states = len(hidden_states)

                layer_idx = int(hidden_layer * no_hidden_states)
                layer_idx = min(max(layer_idx, 1), no_hidden_states) - 1

                selected_hidden = hidden_states[layer_idx]

                # Apply attention-masked pooling to exclude padding tokens
                if (
                    hasattr(feature_inputs, "attention_mask")
                    and feature_inputs.attention_mask is not None
                ):
                    batch_size, hidden_seq_len, hidden_size = selected_hidden.shape
                    device = selected_hidden.device

                    # For Whisper, the attention mask should match the sequence length
                    # If it doesn't, we need to calculate proper lengths
                    if feature_inputs.attention_mask.shape[1] != hidden_seq_len:
                        # Calculate downsample ratio and proper hidden lengths
                        input_lengths = feature_inputs.attention_mask.sum(dim=1)
                        downsample_ratio = (
                            feature_inputs.attention_mask.shape[1] / hidden_seq_len
                        )
                        hidden_lengths = (
                            input_lengths.float() / downsample_ratio
                        ).long()
                        hidden_lengths = torch.clamp(
                            hidden_lengths, min=0, max=hidden_seq_len
                        )

                        # Create attention mask for hidden states
                        seq_range = torch.arange(
                            hidden_seq_len, device=device
                        ).unsqueeze(0)
                        hidden_attention_mask = (
                            seq_range < hidden_lengths.unsqueeze(1)
                        ).to(selected_hidden.dtype)
                    else:
                        # Use the attention mask directly if dimensions match
                        hidden_attention_mask = feature_inputs.attention_mask.to(
                            selected_hidden.dtype
                        )

                    # Apply masked mean pooling
                    hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                    masked_embeddings = selected_hidden * hidden_attention_mask
                    valid_tokens = hidden_attention_mask.sum(dim=1)
                    embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(
                        min=1e-9
                    )
                else:
                    # Fallback to simple mean pooling if no attention mask
                    embeddings = torch.mean(selected_hidden, dim=1)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("WhisperAudioWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


# Model Metas for Different Whisper Models
whisper_langs = [
    "eng-Latn",
    "zho-Hans",
    "deu-Latn",
    "spa-Latn",
    "rus-Cyrl",
    "kor-Hang",
    "fra-Latn",
    "jpn-Jpan",
    "por-Latn",
    "tur-Latn",
    "pol-Latn",
    "cat-Latn",
    "nld-Latn",
    "ara-Arab",
    "swe-Latn",
    "ita-Latn",
    "ind-Latn",
    "hin-Deva",
    "fin-Latn",
    "vie-Latn",
    "heb-Hebr",
    "ukr-Cyrl",
    "ell-Grek",
    "msa-Latn",
    "ces-Latn",
    "ron-Latn",
    "dan-Latn",
    "hun-Latn",
    "tam-Taml",
    "nob-Latn",
    "tha-Thai",
    "urd-Arab",
    "hrv-Latn",
    "bul-Cyrl",
    "lit-Latn",
    "lat-Latn",
    "mri-Latn",
    "mal-Mlym",
    "cym-Latn",
    "slk-Latn",
    "tel-Telu",
    "fas-Arab",
    "lav-Latn",
    "ben-Beng",
    "srp-Cyrl",
    "aze-Latn",
    "slv-Latn",
    "kan-Knda",
    "est-Latn",
    "mkd-Cyrl",
    "bre-Latn",
    "eus-Latn",
    "isl-Latn",
    "hye-Armn",
    "nep-Deva",
    "mon-Cyrl",
    "bos-Latn",
    "kaz-Cyrl",
    "sqi-Latn",
    "swa-Latn",
    "glg-Latn",
    "mar-Deva",
    "pan-Guru",
    "sin-Sinh",
    "khm-Khmr",
    "sna-Latn",
    "yor-Latn",
    "som-Latn",
    "afr-Latn",
    "oci-Latn",
    "kat-Geor",
    "bel-Cyrl",
    "tgk-Cyrl",
    "snd-Arab",
    "guj-Gujr",
    "amh-Ethi",
    "yid-Hebr",
    "lao-Laoo",
    "uzb-Latn",
    "fao-Latn",
    "hat-Latn",
    "pus-Arab",
    "tuk-Latn",
    "nno-Latn",
    "mlt-Latn",
    "san-Deva",
    "ltz-Latn",
    "mya-Mymr",
    "bod-Tibt",
    "tgl-Latn",
    "mlg-Latn",
    "asm-Beng",
    "tat-Cyrl",
    "haw-Latn",
    "lin-Latn",
    "hau-Latn",
    "bak-Cyrl",
    "jav-Latn",
    "sun-Latn",
]


whisper_tiny = ModelMeta(
    loader=WhisperAudioWrapper,
    name="openai/whisper-tiny",
    languages=whisper_langs,
    open_weights=True,
    revision="main",
    release_date="2022-09-27",
    max_tokens=float("inf"),
    n_parameters=39_000_000,
    memory_usage_mb=144,
    embed_dim=512,
    license="mit",
    reference="https://huggingface.co/openai/whisper-tiny",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{radford2022robustspeechrecognitionlargescale,
      title={Robust Speech Recognition via Large-Scale Weak Supervision},
      author={Alec Radford and Jong Wook Kim and Tao Xu and Greg Brockman and Christine McLeavey and Ilya Sutskever},
      year={2022},
      eprint={2212.04356},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2212.04356},
}""",
)

whisper_base = ModelMeta(
    loader=WhisperAudioWrapper,
    name="openai/whisper-base",
    languages=whisper_langs,
    open_weights=True,
    revision="main",
    release_date="2022-09-27",
    max_tokens=float("inf"),
    n_parameters=74_000_000,
    memory_usage_mb=277,
    embed_dim=512,
    license="mit",
    reference="https://huggingface.co/openai/whisper-base",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{radford2022robustspeechrecognitionlargescale,
      title={Robust Speech Recognition via Large-Scale Weak Supervision},
      author={Alec Radford and Jong Wook Kim and Tao Xu and Greg Brockman and Christine McLeavey and Ilya Sutskever},
      year={2022},
      eprint={2212.04356},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2212.04356},
}""",
)

whisper_small = ModelMeta(
    loader=WhisperAudioWrapper,
    name="openai/whisper-small",
    languages=whisper_langs,
    open_weights=True,
    revision="main",
    release_date="2022-09-27",
    max_tokens=float("inf"),
    n_parameters=244_000_000,
    memory_usage_mb=922,
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/openai/whisper-small",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{radford2022robustspeechrecognitionlargescale,
      title={Robust Speech Recognition via Large-Scale Weak Supervision},
      author={Alec Radford and Jong Wook Kim and Tao Xu and Greg Brockman and Christine McLeavey and Ilya Sutskever},
      year={2022},
      eprint={2212.04356},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2212.04356},
}""",
)

whisper_medium = ModelMeta(
    loader=WhisperAudioWrapper,
    name="openai/whisper-medium",
    languages=whisper_langs,
    open_weights=True,
    revision="main",
    release_date="2022-09-27",
    max_tokens=float("inf"),
    n_parameters=769_000_000,
    memory_usage_mb=2914,
    embed_dim=1024,
    license="mit",
    reference="https://huggingface.co/openai/whisper-medium",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{radford2022robustspeechrecognitionlargescale,
      title={Robust Speech Recognition via Large-Scale Weak Supervision},
      author={Alec Radford and Jong Wook Kim and Tao Xu and Greg Brockman and Christine McLeavey and Ilya Sutskever},
      year={2022},
      eprint={2212.04356},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2212.04356},
}""",
)

whisper_large_v3 = ModelMeta(
    loader=WhisperAudioWrapper,
    name="openai/whisper-large-v3",
    languages=whisper_langs,
    open_weights=True,
    revision="main",
    release_date="2022-09-27",
    max_tokens=float("inf"),
    n_parameters=1_550_000_000,
    memory_usage_mb=5887,
    embed_dim=1280,
    license="mit",
    reference="https://huggingface.co/openai/whisper-large-v3",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{radford2022robustspeechrecognitionlargescale,
      title={Robust Speech Recognition via Large-Scale Weak Supervision},
      author={Alec Radford and Jong Wook Kim and Tao Xu and Greg Brockman and Christine McLeavey and Ilya Sutskever},
      year={2022},
      eprint={2212.04356},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2212.04356},
}""",
)



from __future__ import annotations

import heapq
import logging
import shutil
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING, Any

from mteb._create_dataloaders import create_dataloader
from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder, get_prompt
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.types import PromptType

if TYPE_CHECKING:
    from torch.utils.data import DataLoader
    from typing_extensions import Unpack

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import (
        Array,
        BatchedInput,
        CorpusDatasetType,
        EncodeKwargs,
        QueryDatasetType,
        RetrievalOutputType,
        TopRankedDocumentsType,
    )


logger = logging.getLogger(__name__)


class PylateSearchEncoder:
    """Mixin class to add PyLate-based indexing and search to an encoder. Implements [SearchProtocol][mteb.models.SearchProtocol]"""

    base_index_dir: Path | None = None
    _index_dir: Path | None = None
    _index_name: str | None = None
    _index_autodelete: bool = True
    task_corpus: CorpusDatasetType | None = None
    index_kwargs: dict[str, Any] = {}  # noqa: RUF012

    def index(
        self,
        corpus: CorpusDatasetType,
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None,
    ) -> None:
        """Index the corpus for retrieval.

        Args:
            corpus: Corpus dataset to index.
            task_metadata: Metadata of the task, used to determine how to index the corpus.
            hf_split: Split of current task, allows to know some additional information about current split.
            hf_subset: Subset of current task. Similar to `hf_split` to get more information
            encode_kwargs: Additional arguments to pass to the encoder during indexing.
            num_proc: Number of processes to use for indexing.
        """
        self.task_corpus = corpus

        safe_task = task_metadata.name.replace("/", "__")

        index_dir_name = f"mteb-index-{safe_task}-{hf_subset}-{hf_split}"
        if self.base_index_dir is None:
            self._index_dir = Path(tempfile.mkdtemp(prefix=index_dir_name))
        else:
            self._index_dir = self.base_index_dir / index_dir_name
            self._index_dir.mkdir(parents=True, exist_ok=True)

        if self._index_name is None:
            self._index_name = "index"

    def search(
        self,
        queries: QueryDatasetType,
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        top_k: int,
        encode_kwargs: EncodeKwargs,
        top_ranked: TopRankedDocumentsType | None = None,
        num_proc: int | None,
    ) -> RetrievalOutputType:
        queries_dataloader = create_dataloader(
            queries,
            task_metadata,
            prompt_type=PromptType.query,
            batch_size=encode_kwargs.get("batch_size", 32),
            num_proc=num_proc,
        )

        query_embeddings = self._encode(
            queries_dataloader,
            task_metadata=task_metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            prompt_type=PromptType.query,
            **encode_kwargs,
        )
        query_idx_to_id = {i: row["id"] for i, row in enumerate(queries)}

        if top_ranked is not None:
            logger.info("Reranking with PyLate...")
            result_heaps = self._pylate_rerank_documents(
                query_idx_to_id=query_idx_to_id,
                query_embeddings=query_embeddings,
                top_ranked=top_ranked,
                top_k=top_k,
                task_metadata=task_metadata,
                hf_subset=hf_subset,
                hf_split=hf_split,
                encode_kwargs=encode_kwargs,
                num_proc=num_proc,
            )
        else:
            result_heaps = self._pylate_full_corpus_search(
                query_idx_to_id=query_idx_to_id,
                query_embeddings=query_embeddings,
                top_k=top_k,
                task_metadata=task_metadata,
                hf_subset=hf_subset,
                hf_split=hf_split,
                encode_kwargs=encode_kwargs,
                num_proc=num_proc,
            )

        results = {qid: {} for qid in query_idx_to_id.values()}
        for qid in result_heaps:
            for score, corpus_id in result_heaps[qid]:
                results[qid][corpus_id] = score

        return results

    def _pylate_full_corpus_search(
        self,
        query_idx_to_id: dict[int, str],
        query_embeddings: Array,
        task_metadata: TaskMetadata,
        hf_subset: str,
        hf_split: str,
        top_k: int,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None,
    ) -> dict[str, list[tuple[float, str]]]:
        from pylate import indexes, retrieve

        logger.info("Retrieving with MultiVector index...")

        if self._index_dir is None or self._index_name is None:
            raise ValueError("Index path is not set. Call index() before search().")

        index = indexes.PLAID(
            index_folder=str(self._index_dir),
            index_name=self._index_name,
            # disable triton kernel for reproducibility
            # https://github.com/embeddings-benchmark/mteb/pull/3183#issuecomment-3311029707
            use_triton=False,
            **self.index_kwargs,
        )

        # Collect all IDs
        doc_ids = [str(x) for x in self.task_corpus["id"]]

        # Encode entire corpus via dataloader batching
        documents_loader = create_dataloader(
            self.task_corpus,
            task_metadata,
            prompt_type=PromptType.document,
            batch_size=encode_kwargs.get("batch_size", 32),
            num_proc=num_proc,
        )
        documents_embeddings = self._encode(
            documents_loader,
            task_metadata=task_metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            prompt_type=PromptType.document,
            **encode_kwargs,
        )

        # Add documents to index
        index.add_documents(
            documents_ids=doc_ids,
            documents_embeddings=documents_embeddings,
        )
        retriever = retrieve.ColBERT(index=index)
        scores = retriever.retrieve(queries_embeddings=query_embeddings, k=top_k)

        # Build heaps in the same structure as dense path for consistency
        result_heaps = {qid: [] for qid in query_idx_to_id.values()}
        for q_idx, qid in query_idx_to_id.items():
            # scores[q_idx] is a list of dicts: {"id": str, "score": float}
            for item in scores[q_idx]:
                heapq.heappush(
                    result_heaps[qid], (float(item["score"]), str(item["id"]))
                )
        return result_heaps

    def _pylate_rerank_documents(
        self,
        query_idx_to_id: dict[int, str],
        query_embeddings: Array,
        top_ranked: TopRankedDocumentsType,
        top_k: int,
        task_metadata: TaskMetadata,
        hf_subset: str,
        hf_split: str,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None = None,
    ) -> dict[str, list[tuple[float, str]]]:
        """Rerank with PyLate's rank.rerank using per-query candidates.

        Keeps dense rerank untouched by using a PyLate-only path.

        Returns:
            A dictionary mapping query IDs to a list of tuples, each containing a score and a document ID.
        """
        from pylate import rank

        if self.task_corpus is None:
            raise ValueError("Corpus must be set before reranking.")

        result_heaps = {qid: [] for qid in query_idx_to_id.values()}
        doc_id_to_idx = {doc["id"]: idx for idx, doc in enumerate(self.task_corpus)}

        all_doc_embeddings = self._encode(
            create_dataloader(
                self.task_corpus,
                task_metadata,
                prompt_type=PromptType.document,
                batch_size=encode_kwargs.get("batch_size", 32),
                num_proc=num_proc,
            ),
            task_metadata=task_metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            prompt_type=PromptType.document,
            **encode_kwargs,
        )

        # Process one query at a time to keep it simple
        for q_idx, qid in query_idx_to_id.items():
            if qid not in top_ranked:
                continue
            ranked_ids = top_ranked[qid]
            if not ranked_ids:
                continue

            doc_indices = [doc_id_to_idx[doc_id] for doc_id in ranked_ids]
            query_doc_embeddings = [all_doc_embeddings[idx] for idx in doc_indices]

            q_emb = query_embeddings[q_idx]
            reranked = rank.rerank(
                documents_ids=[ranked_ids],
                queries_embeddings=[q_emb],
                documents_embeddings=[query_doc_embeddings],
            )

            # Parse PyLate's output
            for item in reranked[0]:  # list of dicts
                heapq.heappush(
                    result_heaps[qid], (float(item["score"]), str(item["id"]))
                )

            # Keep only top_k
            if len(result_heaps[qid]) > top_k:
                result_heaps[qid] = heapq.nlargest(top_k, result_heaps[qid])

        if self._index_autodelete and self._index_dir is not None:
            try:
                shutil.rmtree(self._index_dir, ignore_errors=True)
            finally:
                self._index_dir = None
                self._index_name = None

        return result_heaps


class MultiVectorModel(PylateSearchEncoder):
    task_corpus: CorpusDatasetType | None = None

    def __init__(
        self,
        model_name: str,
        revision: str | None = None,
        model_prompts: dict[str, str] | None = None,
        index_dir: str | Path | None = None,
        index_name: str | None = None,
        index_autodelete: bool = True,
        index_kwargs: dict[str, Any] | None = None,
        **kwargs,
    ) -> None:
        """Wrapper for MultiVector/ColBERT models (via PyLate)."""
        requires_package(self, "pylate", model_name, "pip install mteb[pylate]")
        from pylate.models import ColBERT  # type: ignore[import]

        self.model_name = model_name
        self.model = ColBERT(self.model_name, revision=revision, **kwargs)
        built_in_prompts = getattr(self.model, "prompts", None)
        if built_in_prompts and not model_prompts:
            self.model_prompts = built_in_prompts
        elif model_prompts and built_in_prompts:
            logger.info(f"Model.prompts will be overwritten with {model_prompts}")
            self.model_prompts = AbsEncoder.validate_task_to_prompt_name(model_prompts)
        elif model_prompts:
            self.model_prompts = AbsEncoder.validate_task_to_prompt_name(model_prompts)
        self.base_index_dir = Path(index_dir) if index_dir else None
        self._index_name = index_name
        self._index_autodelete = index_autodelete
        self.index_kwargs = index_kwargs or {}

    def _encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Unpack[EncodeKwargs],
    ) -> Array:
        prompt = get_prompt(self.model_prompts, task_metadata, prompt_type)
        if prompt:
            logger.info(
                f"Using prompt={prompt} for task={task_metadata.name} prompt_type={prompt_type}"
            )
        else:
            logger.info(
                f"No model prompts found for task={task_metadata.name} prompt_type={prompt_type}"
            )
        logger.debug(f"Encoding {len(inputs)} items.")

        inputs = [text for batch in inputs for text in batch["text"]]

        pred = self.model.encode(
            inputs,
            prompt=prompt,
            is_query=prompt_type == PromptType.query,
            **kwargs,
        )

        return pred


colbert_v2 = ModelMeta(
    loader=MultiVectorModel,
    name="colbert-ir/colbertv2.0",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="c1e84128e85ef755c096a95bdb06b47793b13acf",
    public_training_code=None,
    public_training_data=None,
    release_date="2024-09-21",
    n_parameters=109482240,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=418,
    max_tokens=180,
    embed_dim=None,
    license="mit",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    framework=["PyLate", "ColBERT", "Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/colbert-ir/colbertv2.0",
    use_instructions=False,
    adapted_from=None,
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "mMARCO-NL",
    },
)

jina_colbert_v2 = ModelMeta(
    loader=MultiVectorModel,
    loader_kwargs=dict(
        query_prefix="[QueryMarker]",
        document_prefix="[DocumentMarker]",
        attend_to_expansion_tokens=True,
        trust_remote_code=True,
    ),
    name="jinaai/jina-colbert-v2",
    model_type=["late-interaction"],
    languages=[
        "ara-Arab",
        "ben-Beng",
        "deu-Latn",
        "spa-Latn",
        "eng-Latn",
        "fas-Arab",
        "fin-Latn",
        "fra-Latn",
        "hin-Deva",
        "ind-Latn",
        "jpn-Jpan",
        "kor-Kore",
        "rus-Cyrl",
        "swa-Latn",
        "tel-Telu",
        "tha-Thai",
        "yor-Latn",
        "zho-Hans",
        "nld-Latn",
        "ita-Latn",
        "por-Latn",
        "vie-Latn",
    ],
    open_weights=True,
    revision="4cf816e5e2b03167b132a3c847a9ecd48ba708e1",
    public_training_code=None,
    public_training_data=None,
    release_date="2024-08-16",
    n_parameters=559366144,
    n_embedding_parameters=256004096,
    memory_usage_mb=1067,
    max_tokens=8192,
    embed_dim=None,
    license="cc-by-nc-4.0",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    framework=["PyLate", "ColBERT", "ONNX", "safetensors"],
    reference="https://huggingface.co/jinaai/jina-colbert-v2",
    use_instructions=False,
    adapted_from=None,
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "mMARCO-NL",
        "DuRetrieval",
        "MIRACL",
    },
    citation="""@inproceedings{xiao-etal-2024-jina,
    title = "{J}ina-{C}ol{BERT}-v2: A General-Purpose Multilingual Late Interaction Retriever",
    author = {Jha, Rohan  and
      Wang, Bo  and
      G{\"u}nther, Michael  and
      Mastrapas, Georgios  and
      Sturua, Saba  and
      Mohr, Isabelle  and
      Koukounas, Andreas  and
      Wang, Mohammad Kalim  and
      Wang, Nan  and
      Xiao, Han},
    editor = {S{\"a}lev{\"a}, Jonne  and
      Owodunni, Abraham},
    booktitle = "Proceedings of the Fourth Workshop on Multilingual Representation Learning (MRL 2024)",
    month = nov,
    year = "2024",
    address = "Miami, Florida, USA",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2024.mrl-1.11/",
    doi = "10.18653/v1/2024.mrl-1.11",
    pages = "159--166",
    abstract = "Multi-vector dense models, such as ColBERT, have proven highly effective in information retrieval. ColBERT`s late interaction scoring approximates the joint query-document attention seen in cross-encoders while maintaining inference efficiency closer to traditional dense retrieval models, thanks to its bi-encoder architecture and recent optimizations in indexing and search. In this paper, we introduce a novel architecture and a training framework to support long context window and multilingual retrieval. Leveraging Matryoshka Representation Loss, we further demonstrate that the reducing the embedding dimensionality from 128 to 64 has insignificant impact on the model`s retrieval performance and cut storage requirements by up to 50{\\%}. Our new model, Jina-ColBERT-v2, demonstrates strong performance across a range of English and multilingual retrieval tasks,"
}""",
)


lightonai__gte_moderncolbert_v1 = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/GTE-ModernColBERT-v1",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="c1647d10d6edc6f70837f42f0a978f2df53f51dd",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/gte_modern_colbert.py",
    public_training_data="https://huggingface.co/datasets/lightonai/ms-marco-en-bge-gemma",
    release_date="2025-04-30",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/GTE-ModernColBERT-v1",
    use_instructions=False,
    adapted_from="Alibaba-NLP/gte-modernbert-base",
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "NQ",
        "NQHardNegatives",
        "NanoNQRetrieval",
        "HotpotQA",
        "HotpotQAHardNegatives",
        "CodeSearchNet",
        "FEVER",
        "DBPedia",
        "DBPediaHardNegatives.v2",
        "NanoDBPediaRetrieval",
        "TRECDL2019",
        "TRECDL2020",
        "CornStack",
    },
    citation="""@misc{GTE-ModernColBERT,
    title={GTE-ModernColBERT},
    author={Chaffin, Antoine},
    url={https://huggingface.co/lightonai/GTE-ModernColBERT-v1},
    year={2025}
}""",
)

late_on_code_citation = """@misc{LateOn-Code,
  title  = {LateOn-Code: a Family of State-Of-The-Art Late Interaction Code Retrieval Models},
  author = {Chaffin, Antoine},
  url    = {https://huggingface.co/collections/lightonai/lateon-code},
  year   = {2026}
}"""

lightonai__late_on_code_pretrain = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/LateOn-Code-pretrain",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
        "python-Code",
        "go-Code",
        "java-Code",
        "javascript-Code",
        "ruby-Code",
        "php-Code",
    ],
    open_weights=True,
    revision="71251a6ee61eee488de7e3ae29f5fb4c3c94699b",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/lateon_code/pre-training.py",
    public_training_data="https://huggingface.co/datasets/lightonai/cornstack",
    release_date="2026-02-12",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/LateOn-Code-pretrain",
    use_instructions=False,
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "NQ",
        "NQHardNegatives",
        "NanoNQRetrieval",
        "HotpotQA",
        "HotpotQAHardNegatives",
        "CodeSearchNet",
        "FEVER",
        "DBPedia",
        "DBPediaHardNegatives.v2",
        "NanoDBPediaRetrieval",
        "TRECDL2019",
        "TRECDL2020",
        "CornStack",
    },
    citation=late_on_code_citation,
)


lightonai__late_on_code = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/LateOn-Code",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
        "python-Code",
        "go-Code",
        "java-Code",
        "javascript-Code",
        "ruby-Code",
        "php-Code",
    ],
    open_weights=True,
    revision="734b659a57935ef50562d79581c3ff1f8d825c93",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/lateon_code/fine_tuning.py",
    public_training_data="https://huggingface.co/datasets/lightonai/nv-embed-supervised-distill-dedup-code",
    release_date="2026-02-12",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/LateOn-Code",
    use_instructions=False,
    adapted_from="lightonai/LateOn-Code-pretrain",
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "mMARCO-NL",
        "CornStack",
        "AppsRetrieval",
        "SyntheticText2SQL",
        "CosQA",
        "CodeFeedbackMT",
        "CodeFeedbackST",
        "StackOverflowQA",
        "CodeTransOceanContest",
        "CodeTransOceanDL",
        "CodeSearchNetRetrieval",
        "CodeSearchNetCCRetrieval",
        "COIRCodeSearchNetRetrieval",
        "AppsRetrieval",
        "SyntheticText2SQL",
        "CosQA",
        "CodeFeedbackMT",
        "CodeFeedbackST",
        "StackOverflowQA",
        "CodeTransOceanContest",
        "CodeTransOceanDL",
        "CodeSearchNetRetrieval",
        "CodeSearchNetCCRetrieval",
        "COIRCodeSearchNetRetrieval",
    },
    citation=late_on_code_citation,
)

lightonai__late_on_code_edge_pretrain = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/LateOn-Code-edge-pretrain",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
        "python-Code",
        "go-Code",
        "java-Code",
        "javascript-Code",
        "ruby-Code",
        "php-Code",
    ],
    open_weights=True,
    revision="4ca3a44b3093e72d48461aa6a67cfd5c0025c007",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/lateon_code/pre-training.py",
    public_training_data="https://huggingface.co/datasets/lightonai/cornstack",
    release_date="2026-02-12",
    n_parameters=16797952,
    n_embedding_parameters=12894720,
    memory_usage_mb=64,
    max_tokens=7999,
    embed_dim=48,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/LateOn-Code-edge-pretrain",
    use_instructions=False,
    adapted_from="mixedbread-ai/mxbai-edge-colbert-v0-17m",
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "NQ",
        "HotpotQA",
        "AmazonQA",
        "LoTTE",
        "MultiLongDocRetrieval",
        "CornStack",
    },
    citation=late_on_code_citation,
)


lightonai__late_on_code_edge = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/LateOn-Code-edge",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
        "python-Code",
        "go-Code",
        "java-Code",
        "javascript-Code",
        "ruby-Code",
        "php-Code",
    ],
    open_weights=True,
    revision="07ef20f406c86badca122464808f4cac2f6e4b25",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/lateon_code/fine_tuning.py",
    public_training_data="https://huggingface.co/datasets/lightonai/nv-embed-supervised-distill-dedup-code",
    release_date="2026-02-12",
    n_parameters=16797952,
    n_embedding_parameters=12894720,
    memory_usage_mb=64,
    max_tokens=7999,
    embed_dim=48,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/LateOn-Code-edge",
    use_instructions=False,
    adapted_from="lightonai/LateOn-Code-edge-pretrain",
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "NQ",
        "HotpotQA",
        "AmazonQA",
        "LoTTE",
        "MultiLongDocRetrieval",
        "CornStack",
        "AppsRetrieval",
        "SyntheticText2SQL",
        "CosQA",
        "CodeFeedbackMT",
        "CodeFeedbackST",
        "StackOverflowQA",
        "CodeTransOceanContest",
        "CodeTransOceanDL",
        "CodeSearchNetRetrieval",
        "CodeSearchNetCCRetrieval",
        "COIRCodeSearchNetRetrieval",
    },
    citation=late_on_code_citation,
)


nomic_embed_unsupervised_data = {
    "QuoraRetrieval",
    "CodeSearchNetCCRetrieval",
    "CodeSearchNetRetrieval",
    "COIRCodeSearchNetRetrieval",
}
nomic_embed_supervised_data = {
    "FEVER",
    "FEVERHardNegatives.v2",
    "NanoFEVERRetrieval",
    "MSMARCO",
    "MSMARCOv2",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "HotpotQA",
    "HotpotQAHardNegatives.v2",
    "NanoHotpotQARetrieval",
}
colbert_zero_citation = """@misc{chaffin2026colbertzeropretrainpretraincolbert,
  title         = {ColBERT-Zero: To Pre-train Or Not To Pre-train ColBERT models}, 
  author        = {Antoine Chaffin and Luca Arnaboldi and Amélie Chatelain and Florent Krzakala},
  year          = {2026},
  eprint        = {2602.16609},
  archivePrefix = {arXiv},
  primaryClass  = {cs.CL},
  url           = {https://arxiv.org/abs/2602.16609}, 
}"""
lightonai__colbert_zero_unsupervised = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/ColBERT-Zero-unsupervised",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="d5f1d1e4d7ddb9aa22ce0d8f57f0cd361f39ac77",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/ColBERT-zero/unsupervised.py",
    public_training_data="https://huggingface.co/datasets/nomic-ai/nomic-embed-unsupervised-data",
    release_date="2026-02-19",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/ColBERT-Zero-unsupervised",
    use_instructions=False,
    adapted_from="answerdotai/ModernBERT-base",
    superseded_by=None,
    training_datasets=nomic_embed_unsupervised_data,
    citation=colbert_zero_citation,
)


lightonai__colbert_zero_supervised = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/ColBERT-Zero-supervised",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="9340f7e825b4d37fdf2c6e950c1a928fe24d770b",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/ColBERT-zero/supervised.py",
    public_training_data="https://huggingface.co/datasets/nomic-ai/nomic-embed-supervised",
    release_date="2026-02-19",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/ColBERT-Zero-supervised",
    use_instructions=False,
    adapted_from="lightonai/ColBERT-Zero-unsupervised",
    superseded_by=None,
    training_datasets=nomic_embed_unsupervised_data | nomic_embed_supervised_data,
    citation=colbert_zero_citation,
)

lightonai__colbert_zero = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/ColBERT-Zero",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="5a1f22f2be589f17654d1ed455b635b70d4aff21",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/ColBERT-zero/distillation.py",
    public_training_data="lightonai/ms-marco-en-bge-gemma",
    release_date="2026-02-19",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="apache-2.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/ColBERT-Zero",
    use_instructions=False,
    adapted_from="lightonai/ColBERT-Zero-supervised",
    superseded_by=None,
    training_datasets=nomic_embed_unsupervised_data | nomic_embed_supervised_data,
    citation=colbert_zero_citation,
)

lightonai__reason_moderncolbert = ModelMeta(
    loader=MultiVectorModel,
    name="lightonai/Reason-ModernColBERT",
    model_type=["late-interaction"],
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="a1f75a3d01084caf9d4cea3500a498533352f001",
    public_training_code="https://github.com/lightonai/pylate/blob/main/examples/train/reason_moderncolbert.py",
    public_training_data="https://huggingface.co/datasets/reasonir/reasonir-data",
    release_date="2025-05-22",
    n_parameters=149015808,
    n_embedding_parameters=38684160,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=128,
    license="cc-by-nc-4.0",
    similarity_fn_name="MaxSim",
    framework=["PyLate", "ColBERT", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/lightonai/Reason-ModernColBERT",
    use_instructions=False,
    adapted_from="lightonai/GTE-ModernColBERT-v1",
    superseded_by=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "NQ",
        "NQHardNegatives",
        "NanoNQRetrieval",
        "HotpotQA",
        "HotpotQAHardNegatives",
        "CodeSearchNet",
        "FEVER",
        "DBPedia",
        "DBPediaHardNegatives.v2",
        "NanoDBPediaRetrieval",
        "TRECDL2019",
        "TRECDL2020",
        "CornStack",
    },
    citation="""@misc{Reason-ModernColBERT,
title={Reason-ModernColBERT},
author={Chaffin, Antoine},
url={https://huggingface.co/lightonai/Reason-ModernColBERT},
year={2025}
}""",
)



from .fa_mteb_clustering import (
    BeytooteClustering,
    DigikalamagClustering,
    HamshahriClustring,
    NLPTwitterAnalysisClustering,
    SIDClustring,
)

__all__ = [
    "BeytooteClustering",
    "DigikalamagClustering",
    "HamshahriClustring",
    "NLPTwitterAnalysisClustering",
    "SIDClustring",
]



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class TwitterSemEval2015PC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="TwitterSemEval2015",
        dataset={
            "path": "mteb/twittersemeval2015-pairclassification",
            "revision": "70970daeab8776df92f5ea462b6173c0b46fd2d1",
        },
        description="Paraphrase-Pairs of Tweets from the SemEval 2015 workshop.",
        reference="https://alt.qcri.org/semeval2015/task1/",
        category="t2t",
        modalities=["text"],
        type="PairClassification",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=("2015-01-01", "2015-12-31"),  # publication year
        domains=["Social", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{xu-etal-2015-semeval,
  address = {Denver, Colorado},
  author = {Xu, Wei  and
Callison-Burch, Chris  and
Dolan, Bill},
  booktitle = {Proceedings of the 9th International Workshop on Semantic Evaluation ({S}em{E}val 2015)},
  doi = {10.18653/v1/S15-2001},
  editor = {Nakov, Preslav  and
Zesch, Torsten  and
Cer, Daniel  and
Jurgens, David},
  month = jun,
  pages = {1--11},
  publisher = {Association for Computational Linguistics},
  title = {{S}em{E}val-2015 Task 1: Paraphrase and Semantic Similarity in {T}witter ({PIT})},
  url = {https://aclanthology.org/S15-2001},
  year = {2015},
}
""",
        prompt="Retrieve tweets that are semantically similar to the given tweet",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.rename_column("sent1", "sentence1")
        self.dataset = self.dataset.rename_column("sent2", "sentence2")



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class LLaVAIT2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LLaVAIT2TRetrieval",
        description="Retrieve responses to answer questions about images.",
        reference="https://github.com/LinWeizheDragon/FLMR/blob/main/docs/Datasets.md",
        dataset={
            "path": "izhx/UMRB-LLaVA",
            "revision": "2a5ed414aab388d8cdd244ba2cf8c8960df4d44d",
        },
        type="Any2AnyRetrieval",
        category="it2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2024-07-06", "2024-02-26"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{lin-etal-2024-preflmr,
  address = {Bangkok, Thailand},
  author = {Lin, Weizhe  and
Mei, Jingbiao  and
Chen, Jinghong  and
Byrne, Bill},
  booktitle = {Proceedings of the 62nd Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)},
  doi = {10.18653/v1/2024.acl-long.289},
  editor = {Ku, Lun-Wei  and
Martins, Andre  and
Srikumar, Vivek},
  month = aug,
  pages = {5294--5316},
  publisher = {Association for Computational Linguistics},
  title = {{P}re{FLMR}: Scaling Up Fine-Grained Late-Interaction Multi-modal Retrievers},
  url = {https://aclanthology.org/2024.acl-long.289},
  year = {2024},
}
""",
        prompt={
            "query": "Provide a specific decription of the image along with the following question."
        },
    )



from mteb.abstasks.clustering import (
    AbsTaskClustering,
)
from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata


class AlloProfClusteringS2S(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="AlloProfClusteringS2S",
        description="Clustering of document titles from Allo Prof dataset. Clustering of 10 sets on the document topic.",
        reference="https://huggingface.co/datasets/lyon-nlp/alloprof",
        dataset={
            "path": "mteb/AlloProfClusteringS2S",
            "revision": "65e8374374d699feeffb65012a42e033d48ede38",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fra-Latn"],
        main_score="v_measure",
        date=("1996-01-01", "2023-04-14"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{lef23,
  author = {Lefebvre-Brossard, Antoine and Gazaille, Stephane and Desmarais, Michel C.},
  copyright = {Creative Commons Attribution Non Commercial Share Alike 4.0 International},
  doi = {10.48550/ARXIV.2302.07738},
  keywords = {Computation and Language (cs.CL), Information Retrieval (cs.IR), Machine Learning (cs.LG), FOS: Computer and information sciences, FOS: Computer and information sciences},
  publisher = {arXiv},
  title = {Alloprof: a new French question-answer education dataset and its use in an information retrieval case study},
  url = {https://arxiv.org/abs/2302.07738},
  year = {2023},
}
""",
        superseded_by="AlloProfClusteringS2S.v2",
    )


class AlloProfClusteringS2SFast(AbsTaskClustering):
    max_depth = 1
    max_document_to_embed = 2556
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="AlloProfClusteringS2S.v2",
        description="Clustering of document titles from Allo Prof dataset. Clustering of 10 sets on the document topic.",
        reference="https://huggingface.co/datasets/lyon-nlp/alloprof",
        dataset={
            "path": "mteb/AlloProfClusteringS2S.v2",
            "revision": "c802b993cf2a8dcaeadb6f9cc8ccd364e251feaa",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fra-Latn"],
        main_score="v_measure",
        # (date of founding of the dataset source site, date of dataset paper publication)
        date=("1996-01-01", "2023-04-14"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{lef23,
  author = {Lefebvre-Brossard, Antoine and Gazaille, Stephane and Desmarais, Michel C.},
  copyright = {Creative Commons Attribution Non Commercial Share Alike 4.0 International},
  doi = {10.48550/ARXIV.2302.07738},
  keywords = {Computation and Language (cs.CL), Information Retrieval (cs.IR), Machine Learning (cs.LG), FOS: Computer and information sciences, FOS: Computer and information sciences},
  publisher = {arXiv},
  title = {Alloprof: a new French question-answer education dataset and its use in an information retrieval case study},
  url = {https://arxiv.org/abs/2302.07738},
  year = {2023},
}
""",
        adapted_from=["AlloProfClusteringS2S"],
    )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class EuroSATZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="EuroSATZeroShot",
        description="Classifying satellite images.",
        reference="https://ieeexplore.ieee.org/document/8736785",
        dataset={
            "path": "mteb/eurosat-rgb",
            "revision": "da02b77448e750b3b034e0b5da008167b9706571",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2019-01-01",
            "2019-03-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Scene recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@article{8736785,
  author = {Helber, Patrick and Bischke, Benjamin and Dengel, Andreas and Borth, Damian},
  doi = {10.1109/JSTARS.2019.2918242},
  journal = {IEEE Journal of Selected Topics in Applied Earth Observations and Remote Sensing},
  keywords = {Satellites;Earth;Remote sensing;Machine learning;Spatial resolution;Feature extraction;Benchmark testing;Dataset;deep convolutional neural network;deep learning;earth observation;land cover classification;land use classification;machine learning;remote sensing;satellite image classification;satellite images},
  number = {7},
  pages = {2217-2226},
  title = {EuroSAT: A Novel Dataset and Deep Learning Benchmark for Land Use and Land Cover Classification},
  volume = {12},
  year = {2019},
}
""",
    )

    def get_candidate_labels(self) -> list[str]:
        labels = [
            "annual crop land",
            "forest land",
            "brushland or shrubland",
            "highway or road",
            "industrial land",
            "pasture land",
            "permanent crop land",
            "residential land",
            "river",
            "sea or lake",
        ]
        return [f"a centered satellite photo of {name}." for name in labels]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MusicCapsA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MusicCapsA2TRetrieval",
        description="Natural language description for music audio.",
        reference="https://github.com/nateraw/download-musiccaps-dataset",
        dataset={
            "path": "mteb/MusicCaps_a2t",
            "revision": "fb2fffc9a93f729d1844dfac3f8c16386fbe226b",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["audio", "text"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="hit_rate_at_5",
        date=("2023-01-01", "2023-12-31"),
        domains=["Music"],
        task_subtypes=["Music Caption Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{agostinelli2023musiclmgeneratingmusictext,
  archiveprefix = {arXiv},
  author = {Andrea Agostinelli and Timo I. Denk and Zalán Borsos and Jesse Engel and Mauro Verzetti and Antoine Caillon and Qingqing Huang and Aren Jansen and Adam Roberts and Marco Tagliasacchi and Matt Sharifi and Neil Zeghidour and Christian Frank},
  eprint = {2301.11325},
  primaryclass = {cs.SD},
  title = {MusicLM: Generating Music From Text},
  url = {https://arxiv.org/abs/2301.11325},
  year = {2023},
}
""",
    )


class MusicCapsT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MusicCapsT2ARetrieval",
        description="Natural language description for music audio.",
        reference="https://github.com/nateraw/download-musiccaps-dataset",
        dataset={
            "path": "mteb/MusicCaps_t2a",
            "revision": "995f536066b2bb7044f6b454f24a76a1c95caa33",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["audio", "text"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="hit_rate_at_5",
        date=("2023-01-01", "2023-12-31"),
        domains=["Music"],
        task_subtypes=["Music Caption Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{agostinelli2023musiclmgeneratingmusictext,
  archiveprefix = {arXiv},
  author = {Andrea Agostinelli and Timo I. Denk and Zalán Borsos and Jesse Engel and Mauro Verzetti and Antoine Caillon and Qingqing Huang and Aren Jansen and Adam Roberts and Marco Tagliasacchi and Matt Sharifi and Neil Zeghidour and Christian Frank},
  eprint = {2301.11325},
  primaryclass = {cs.SD},
  title = {MusicLM: Generating Music From Text},
  url = {https://arxiv.org/abs/2301.11325},
  year = {2023},
}
""",
    )



from __future__ import annotations

import logging
import tempfile
import warnings
from pathlib import Path
from typing import TYPE_CHECKING, Any, Literal, cast

import huggingface_hub
import numpy as np
import pandas as pd
from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import overload

from mteb.types import Modalities

from .task_result import TaskError, TaskResult

if TYPE_CHECKING:
    from collections.abc import Callable, Iterable

    from mteb.abstasks.abstask import AbsTask
    from mteb.abstasks.task_metadata import (
        TaskDomain,
        TaskType,
    )
    from mteb.types import (
        ISOLanguage,
        ISOLanguageScript,
        Score,
        ScoresDict,
        SplitName,
    )


logger = logging.getLogger(__name__)


def _aggregate_and_pivot(
    df: pd.DataFrame,
    columns: list[str],
    aggregation_level: Literal["subset", "split", "task", "language"],
    format: Literal["wide", "long"],
    aggregation_fn: Callable[[list[Score]], Any] | None,
) -> pd.DataFrame:
    if aggregation_level == "subset":
        index_columns = ["task_name", "split", "subset"]

    elif aggregation_level == "split":
        index_columns = ["task_name", "split"]

    elif aggregation_level == "task":
        index_columns = ["task_name"]

    elif aggregation_level == "language":
        index_columns = ["language"]
        df = df.explode("language").reset_index(
            drop=True
        )  # each language in its own row before aggregation

    # perform aggregation
    if aggregation_fn is None:
        aggregation_fn = np.mean

    if format == "wide":
        return df.pivot_table(
            index=index_columns,
            columns=columns,
            values="score",
            aggfunc=aggregation_fn,  # type: ignore[arg-type]
        ).reset_index()
    elif format == "long":
        return (
            df.groupby(columns + index_columns)
            .agg(score=("score", aggregation_fn))
            .reset_index()
        )


class ModelResult(BaseModel):
    """Data class to hold the results of a model on a set of tasks.

    Attributes:
        model_name: Name of the model.
        model_revision: Revision of the model.
        task_results: List of TaskResult objects.
    """

    model_name: str
    model_revision: str | None
    task_results: list[TaskResult]
    default_modalities: list[Modalities] = Field(
        default_factory=lambda: [cast("Modalities", "text")], alias="modalities"
    )
    model_config = (
        ConfigDict(  # to free up the name model_* which is otherwise protected
            protected_namespaces=(),
        )
    )
    exceptions: list[TaskError] | None = None
    experiment_name: str | None = None

    def __repr__(self) -> str:
        n_entries = len(self.task_results)
        return (
            f"ModelResult(model_name='{self.model_name}', model_revision='{self.model_revision}', "
            f"{'experiment_name=' + self.experiment_name + ', ' if self.experiment_name else ''}"
            f"task_results=[...](#{n_entries}), ...)"
        )

    @classmethod
    def from_validated(cls, **data: dict[str, Any]) -> ModelResult:
        """Create a ModelResult from validated data.

        Args:
            data: The validated data.
        """
        data["task_results"] = [  # type: ignore[assignment]
            TaskResult.from_validated(**res)  # type: ignore[arg-type]
            for res in data["task_results"]
        ]
        return cls.model_construct(**data)  # type: ignore[arg-type]

    def _filter_tasks(
        self,
        task_names: list[str] | None = None,
        languages: list[str] | None = None,
        domains: list[TaskDomain] | None = None,
        task_types: list[TaskType] | None = None,
        modalities: list[Modalities] | None = None,
        is_public: bool | None = None,
    ) -> ModelResult:
        new_task_results = []
        for task_result in self.task_results:
            if (task_names is not None) and (task_result.task_name not in task_names):
                continue
            if languages is not None:
                task_languages = task_result.languages
                if not any(lang in task_languages for lang in languages):
                    continue
            if domains is not None:
                task_domains = task_result.domains
                if not any(domain in task_domains for domain in domains):
                    continue
            if (task_types is not None) and (task_result.task_type not in task_types):
                continue
            if modalities is not None:
                task_modalities = getattr(task_result, "modalities", [])
                if not any(modality in task_modalities for modality in modalities):
                    continue
            if (is_public is not None) and (task_result.is_public is not is_public):
                continue
            new_task_results.append(task_result)
        return type(self).model_construct(
            model_name=self.model_name,
            model_revision=self.model_revision,
            task_results=new_task_results,
            experiment_name=self.experiment_name,
        )

    def select_tasks(self, tasks: Iterable[AbsTask]) -> ModelResult:
        """Select tasks from the ModelResult based on a list of AbsTask objects.

        Args:
            tasks: A sequence of AbsTask objects to select from the ModelResult.
        """
        task_name_to_task = {task.metadata.name: task for task in tasks}
        new_task_results = [
            task_res.validate_and_filter_scores(task_name_to_task[task_res.task_name])
            for task_res in self.task_results
            if task_res.task_name in task_name_to_task
        ]
        return type(self).model_construct(
            model_name=self.model_name,
            model_revision=self.model_revision,
            task_results=new_task_results,
            experiment_name=self.experiment_name,
        )

    @overload
    def _get_scores(
        self,
        splits: list[SplitName] | None = None,
        languages: list[ISOLanguage | ISOLanguageScript] | None = None,
        scripts: list[ISOLanguageScript] | None = None,
        getter: Callable[[ScoresDict], Score] | None = None,
        aggregation: Callable[[list[Score]], Any] | None = None,
        format: Literal["wide"] = "wide",
    ) -> dict: ...

    @overload
    def _get_scores(
        self,
        splits: list[SplitName] | None = None,
        languages: list[ISOLanguage | ISOLanguageScript] | None = None,
        scripts: list[ISOLanguageScript] | None = None,
        getter: Callable[[ScoresDict], Score] | None = None,
        aggregation: Callable[[list[Score]], Any] | None = None,
        format: Literal["long"] = "long",
    ) -> list: ...

    def _get_scores(
        self,
        splits: list[SplitName] | None = None,
        languages: list[ISOLanguage | ISOLanguageScript] | None = None,
        scripts: list[ISOLanguageScript] | None = None,
        getter: Callable[[ScoresDict], Score] | None = None,
        aggregation: Callable[[list[Score]], Any] | None = None,
        format: Literal["wide", "long"] = "wide",
    ) -> dict | list:
        if (getter is not None) or (aggregation is not None) or (scripts is not None):
            use_fast = False
            getter = (
                getter if getter is not None else lambda scores: scores["main_score"]
            )
            aggregation = aggregation if aggregation is not None else np.mean
        else:
            use_fast = True
        aggregation = cast("Callable[[list[Score]], Any]", aggregation)
        getter = cast("Callable[[ScoresDict], Score]", getter)

        if format == "wide":
            scores = {}
            for res in self.task_results:
                try:
                    if use_fast:
                        scores[res.task_name] = res._get_score_fast(
                            splits=splits,
                            languages=languages,
                        )
                    else:
                        scores[res.task_name] = res.get_score(
                            splits=splits,
                            languages=languages,
                            aggregation=aggregation,
                            getter=getter,
                            scripts=scripts,
                        )
                except Exception as e:
                    warnings.warn(
                        f"Couldn't get scores for {res.task_name} due to {e}."
                    )
            return scores
        if format == "long":
            entries = []
            for task_res in self.task_results:
                try:
                    if use_fast:
                        score = task_res._get_score_fast(
                            splits=splits,
                            languages=languages,
                        )
                    else:
                        score = task_res.get_score(
                            splits=splits,
                            languages=languages,
                            aggregation=aggregation,
                            getter=getter,
                            scripts=scripts,
                        )
                    entry = dict(
                        model_name=self.model_name,
                        model_revision=self.model_revision,
                        task_name=task_res.task_name,
                        score=score,
                        mteb_version=task_res.mteb_version,
                        dataset_revision=task_res.dataset_revision,
                        evaluation_time=task_res.evaluation_time,
                        kg_co2_emissions=task_res.kg_co2_emissions,
                    )
                    entries.append(entry)
                except Exception as e:
                    warnings.warn(
                        f"Couldn't get scores for {task_res.task_name} due to {e}."
                    )
            return entries

    def _get_score_for_table(self) -> list[dict[str, str | float | list[str]]]:
        scores_data = []
        model_name = self.model_name
        for task_result in self.task_results:
            task_name = task_result.task_name
            for split, scores_list in task_result.scores.items():
                for score_item in scores_list:
                    row = {
                        "model_name": model_name,
                        "model_revision": self.model_revision,
                        "task_name": task_name,
                        "split": split,
                        "language": score_item.get("languages", ["Unknown"]),
                        "subset": score_item.get("hf_subset", "default"),
                        "score": score_item.get("main_score", None),
                    }
                    scores_data.append(row)

        return scores_data

    def to_dataframe(
        self,
        aggregation_level: Literal["subset", "split", "task"] = "task",
        aggregation_fn: Callable[[list[Score]], Any] | None = None,
        include_model_revision: bool = False,
        format: Literal["wide", "long"] = "wide",
    ) -> pd.DataFrame:
        """Get a DataFrame with the scores for all models and tasks.

        The DataFrame will have the following columns in addition to the metadata columns:

        - model_name: The name of the model.
        - task_name: The name of the task.
        - score: The main score of the model on the task.

        In addition, the DataFrame can have the following columns depending on the aggregation level:

        - split: The split of the task. E.g. "test", "train", "validation".
        - subset: The subset of the task. E.g. "en", "fr-en".

        Afterwards, the DataFrame will be aggregated according to the aggregation method and pivoted to either a wide format.

        Args:
            aggregation_level: The aggregation to use. Can be one of:
                - "subset"/None: No aggregation will be done. The DataFrame will have one row per model, task, split and subset.
                - "split": Aggregates the scores by split. The DataFrame will have one row per model, task and split.
                - "task": Aggregates the scores by task. The DataFrame will have one row per model and task.
            aggregation_fn: The function to use for aggregation. If None, the mean will be used.
            include_model_revision: If True, the model revision will be included in the DataFrame. If False, it will be excluded.
            format: The format of the DataFrame. Can be one of:
                - "wide": The DataFrame will be of shape (number of tasks, number of models). Scores will be in the cells.
                - "long": The DataFrame will of length (number of tasks * number of model). Scores will be in columns.

        Returns:
            A DataFrame with the scores for all models and tasks.
        """
        scores_data = self._get_score_for_table()

        if not scores_data:
            msg = "No scores data available. Returning empty DataFrame."
            logger.warning(msg)
            warnings.warn(msg)
            return pd.DataFrame()

        # Create DataFrame
        df = pd.DataFrame(scores_data)

        _columns = ["model_name"]
        if include_model_revision is False:
            df = df.drop(columns=["model_revision"])
        else:
            _columns.append("model_revision")

        return _aggregate_and_pivot(
            df,
            columns=_columns,
            aggregation_level=aggregation_level,
            format=format,
            aggregation_fn=aggregation_fn,
        )

    def __hash__(self) -> int:
        return id(self)

    def __iter__(self) -> Iterable[TaskResult]:  # type: ignore[override]
        return iter(self.task_results)

    def __getitem__(self, index) -> TaskResult:
        return self.task_results[index]

    def __len__(self) -> int:
        return len(self.task_results)

    @property
    def languages(self) -> list[str]:
        """Get all languages in the model results.

        Returns:
            A list of languages in the model results.
        """
        langs = []
        for task_res in self.task_results:
            langs.extend(task_res.languages)
        return list(set(langs))

    @property
    def domains(self) -> list[str]:
        """Get all domains in the model results.

        Returns:
            A list of domains in the model results.

        """
        ds = []
        for task_res in self.task_results:
            ds.extend(task_res.domains)
        return list(set(ds))

    @property
    def task_types(self) -> list[str]:
        """Get all task types in the model results.

        Returns:
            A list of task types in the model results.
        """
        return list({task_res.task_type for task_res in self.task_results})

    @property
    def task_names(self) -> list[str]:
        """Get all task names in the model results.

        Returns:
            A list of task names in the model results.
        """
        return [task_res.task_name for task_res in self.task_results]

    @property
    def modalities(self) -> list[Modalities]:
        """Get all modalities in the task results.

        Returns:
            A list of modalities in the task results.
        """
        mods: list[Modalities] = []
        for task_res in self.task_results:
            task_modalities = getattr(task_res, "modalities", [])
            mods.extend(task_modalities)
        if not mods:
            mods = self.default_modalities
        return list(set(mods))

    def to_disk(self, path: Path) -> None:
        """Save ModelResult to disk as JSON.

        Args:
            path: The path to the file to save.
        """
        with path.open("w") as f:
            f.write(self.model_dump_json(indent=2))

    @classmethod
    def from_disk(cls, path: Path) -> ModelResult:
        """Load ModelResult from disk.

        Args:
            path: The path to the JSON file to load.

        Returns:
            The loaded ModelResult object.
        """
        with path.open("r", encoding="utf-8") as f:
            return cls.model_validate_json(f.read())

    def push_model_results(
        self, user: str | None = None, *, create_pr: bool = False
    ) -> None:
        """Push the model results to the Hugging Face Hub.

        Args:
            user: The user or organization of results source.
            create_pr: Whether to create a pull request
        """
        with tempfile.TemporaryDirectory() as tmpdir:
            path = Path(tmpdir)
            for task_result in self.task_results:
                task_results = task_result._to_hf_benchmark_result(user)
                with (path / f"{task_result.task_name}.yaml").open(
                    "w", encoding="utf-8"
                ) as f:
                    f.write(task_results.to_yaml())

            huggingface_hub.upload_folder(
                repo_id=self.model_name,
                repo_type="model",
                path_in_repo=".eval_results",
                folder_path=path,
                commit_message=f"Add evaluation results for model {self.model_name} revision {self.model_revision}",
                create_pr=create_pr,
            )



from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata


class SpanishNewsClusteringP2P(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="SpanishNewsClusteringP2P",
        description="Clustering of news articles, 7 topics in total.",
        reference="https://www.kaggle.com/datasets/kevinmorgado/spanish-news-classification",
        dataset={
            "path": "jinaai/spanish_news_clustering",
            "revision": "bf8ca8ddc5b7da4f7004720ddf99bbe0483480e6",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["spa-Latn"],
        main_score="v_measure",
        date=("2017-01-01", "2019-12-31"),
        domains=["News", "Written"],
        task_subtypes=["Thematic clustering"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{kevinmorgado2019spanish,
  author = {Kevin Morgado},
  howpublished = {Kaggle},
  title = {Spanish News Classification},
  url = {https://www.kaggle.com/datasets/kevinmorgado/spanish-news-classification},
  year = {2019},
}
""",
    )



from __future__ import annotations

import logging
from collections import defaultdict
from typing import TYPE_CHECKING, Any, ClassVar, TypedDict, cast

from datasets import Dataset, DatasetDict
from sklearn.metrics import accuracy_score, f1_score, precision_score, recall_score

from mteb._evaluators import BitextMiningEvaluator
from mteb.abstasks._statistics_calculation import calculate_text_statistics
from mteb.abstasks.abstask import AbsTask
from mteb.models import EncoderProtocol
from mteb.types.statistics import SplitDescriptiveStatistics

if TYPE_CHECKING:
    from pathlib import Path

    from mteb.models import MTEBModels
    from mteb.types import EncodeKwargs, HFSubset, ScoresDict
    from mteb.types.statistics import TextStatistics

logger = logging.getLogger(__name__)


class BitextDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for Bitext

    Attributes:
        num_samples: number of samples in the dataset.
        number_of_characters: Total number of symbols in the dataset.
        unique_pairs: Number of duplicate pairs

        sentence1_statistics: Statistics for sentence1
        sentence2_statistics: Statistics for sentence2
    """

    num_samples: int
    number_of_characters: int
    unique_pairs: int

    sentence1_statistics: TextStatistics
    sentence2_statistics: TextStatistics


class BitextMiningMetrics(TypedDict):
    """Metrics for BitextMining tasks

    Attributes:
        precision: Precision of the model.
        recall: Recall of the model.
        f1: F1 score of the model.
        accuracy: Accuracy of the model.
    """

    precision: float
    recall: float
    f1: float
    accuracy: float


class AbsTaskBitextMining(AbsTask):
    """Abstract class for BitextMining tasks

    The similarity is computed between pairs and the results are ranked.

    Attributes:
        dataset: A HuggingFace dataset containing the data for the task. It must contain the following columns sentence1 and sentence2 for the two texts to be compared.
        parallel_subsets: If true task language pairs should be in one split as column names, otherwise each language pair should be a subset.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    parallel_subsets = False
    abstask_prompt = "Retrieve parallel sentences."
    _DEFAULT_PAIR: ClassVar[list[tuple[str, str]]] = [("sentence1", "sentence2")]

    def evaluate(
        self,
        model: MTEBModels,
        split: str = "test",
        subsets_to_run: list[HFSubset] | None = None,
        *,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> dict[HFSubset, ScoresDict]:
        """Added load for "parallel" datasets"""
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        if not self.data_loaded:
            self.load_data(num_proc=num_proc)

        hf_subsets = self.hf_subsets

        # If subsets_to_run is specified, filter the hf_subsets accordingly
        if subsets_to_run is not None:
            hf_subsets = [s for s in hf_subsets if s in subsets_to_run]

        encoder_model = cast("EncoderProtocol", model)

        if self.dataset is None:
            raise ValueError("Dataset is not loaded.")

        scores: dict[str, BitextMiningMetrics] = {}
        if self.parallel_subsets:
            scores = self._evaluate_subset(  # type: ignore[assignment]
                encoder_model,
                self.dataset[split],
                parallel=True,
                hf_split=split,
                hf_subset="parallel",
                encode_kwargs=encode_kwargs,
                prediction_folder=prediction_folder,
                num_proc=num_proc,
                **kwargs,
            )
        else:
            for hf_subset in hf_subsets:
                logger.info(
                    f"Task: {self.metadata.name}, split: {split}, subset: {hf_subset}. Running..."
                )

                if hf_subset not in self.dataset and hf_subset == "default":
                    data_split = self.dataset[split]
                else:
                    data_split = self.dataset[hf_subset][split]
                scores[hf_subset] = self._evaluate_subset(  # type: ignore[assignment]
                    encoder_model,
                    data_split,
                    hf_split=split,
                    hf_subset=hf_subset,
                    encode_kwargs=encode_kwargs,
                    prediction_folder=prediction_folder,
                    num_proc=num_proc,
                    **kwargs,
                )

        return cast("dict[HFSubset, ScoresDict]", scores)

    def _get_pairs(self, parallel: bool) -> list[tuple[str, str]]:
        pairs = self._DEFAULT_PAIR
        if parallel:
            pairs = [langpair.split("-") for langpair in self.hf_subsets]  # type: ignore[misc]
        return pairs

    def _evaluate_subset(  # type: ignore[override]
        self,
        model: EncoderProtocol,
        data_split: Dataset,
        *,
        hf_split: str,
        hf_subset: str,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        parallel: bool = False,
        num_proc: int | None = None,
        **kwargs,
    ) -> BitextMiningMetrics | dict[str, BitextMiningMetrics]:
        pairs = self._get_pairs(parallel)

        evaluator = BitextMiningEvaluator(
            data_split,
            task_metadata=self.metadata,
            pair_columns=pairs,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **kwargs,
        )
        # NOTE: used only by BUCC
        gold = (
            list(zip(range(len(data_split)), range(len(data_split))))
            if "gold" not in data_split
            else data_split["gold"]
        )

        neighbours = evaluator(model, encode_kwargs=encode_kwargs, num_proc=num_proc)

        if prediction_folder:
            self._save_task_predictions(
                neighbours,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        if parallel:
            parallel_metrics = {}
            for keys, nearest_neighbors in neighbours.items():
                parallel_metrics[keys] = self._compute_metrics(nearest_neighbors, gold)

            for v in parallel_metrics.values():
                self._add_main_score(v)
            return parallel_metrics
        def_pair_str = "-".join(self._DEFAULT_PAIR[0])
        metrics = self._compute_metrics(neighbours[def_pair_str], gold)
        self._add_main_score(metrics)
        return metrics

    def _compute_metrics(
        self,
        nearest_neighbors: list[dict[str, float]],
        gold: list[tuple[int, int]],
    ) -> BitextMiningMetrics:
        logger.info("Computing metrics...")
        labels = []
        predictions = []
        for i, x in enumerate(nearest_neighbors):
            j = x["corpus_id"]
            predictions.append(j)
            labels.append(gold[i][1])

        return BitextMiningMetrics(
            precision=precision_score(
                labels, predictions, zero_division=0, average="weighted"
            ),
            recall=recall_score(
                labels, predictions, zero_division=0, average="weighted"
            ),
            f1=f1_score(labels, predictions, zero_division=0, average="weighted"),
            accuracy=accuracy_score(labels, predictions),
        )

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> BitextDescriptiveStatistics:
        pairs_cols = self._get_pairs(self.parallel_subsets)
        if hf_subset:
            if self.parallel_subsets:
                sent_1, sent_2 = hf_subset.split("-")
                sentence1 = self.dataset[split][sent_1]
                sentence2 = self.dataset[split][sent_2]
            else:
                sent_1, sent_2 = pairs_cols[0]
                sentence1 = self.dataset[hf_subset][split][sent_1]
                sentence2 = self.dataset[hf_subset][split][sent_2]
        elif compute_overall:
            sentence1, sentence2 = [], []
            if self.parallel_subsets:
                for hf_subset in self.metadata.eval_langs:
                    sent_1, sent_2 = hf_subset.split("-")
                    sentence1.extend(self.dataset[split][sent_1])
                    sentence2.extend(self.dataset[split][sent_2])
            else:
                sent_1, sent_2 = pairs_cols[0]
                for hf_subset in self.metadata.eval_langs:
                    sentence1.extend(self.dataset[hf_subset][split][sent_1])
                    sentence2.extend(self.dataset[hf_subset][split][sent_2])
        else:
            sent_1, sent_2 = pairs_cols[0]
            sentence1 = self.dataset[split][sent_1]
            sentence2 = self.dataset[split][sent_2]

        text1_statistics = calculate_text_statistics(sentence1)
        text2_statistics = calculate_text_statistics(sentence2)
        unique_pairs = len(set(zip(sentence1, sentence2)))

        return BitextDescriptiveStatistics(
            num_samples=len(sentence1),
            number_of_characters=(
                text1_statistics["total_text_length"]
                + text2_statistics["total_text_length"]
            ),
            unique_pairs=unique_pairs,
            sentence1_statistics=text1_statistics,
            sentence2_statistics=text2_statistics,
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        if self.dataset is None:
            raise ValueError("Dataset is not loaded.")

        if self.metadata.is_multilingual:
            dataset: dict[str, dict[str, list[str]]] = defaultdict(dict)
            for config in self.metadata.eval_langs:
                logger.info(f"Converting {config} of {self.metadata.name}")

                if self.parallel_subsets:
                    for split in self.dataset:
                        sent_1, sent_2 = config.split("-")
                        dataset[split][sent_1] = self.dataset[split][sent_1]
                        dataset[split][sent_2] = self.dataset[split][sent_2]
                else:
                    sent_1, sent_2 = self._get_pairs(self.parallel_subsets)[0]
                    lang_1, lang_2 = config.split("-")
                    for split in self.dataset[config]:
                        dataset[split][lang_1] = self.dataset[config][split][sent_1]
                        dataset[split][lang_2] = self.dataset[config][split][sent_2]
            dataset_dict = DatasetDict(
                {split: Dataset.from_dict(dataset[split]) for split in dataset}
            )
            dataset_dict.push_to_hub(repo_name, num_proc=num_proc)
        else:
            sentences = {}
            for split in self.dataset:
                sent_1, sent_2 = self._get_pairs(self.parallel_subsets)[0]
                sentences[split] = Dataset.from_dict(
                    {
                        "sentence1": self.dataset[split][sent_1],
                        "sentence2": self.dataset[split][sent_2],
                    }
                )
            sentences = DatasetDict(sentences)
            sentences.push_to_hub(repo_name, num_proc=num_proc)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "fas": ["fas-Arab"],
    "rus": ["rus-Cyrl"],
    "zho": ["zho-Hans"],
}


class NeuCLIR2023Retrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NeuCLIR2023Retrieval",
        description="The task involves identifying and retrieving the documents that are relevant to the queries.",
        reference="https://neuclir.github.io/",
        dataset={
            "path": "mteb/NeuCLIR2022Retrieval",
            "revision": "95cad8671c1c31908766a689755c307f4770411f",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_20",
        date=("2022-08-01", "2023-06-30"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="odc-by",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{lawrie2024overview,
  archiveprefix = {arXiv},
  author = {Dawn Lawrie and Sean MacAvaney and James Mayfield and Paul McNamee and Douglas W. Oard and Luca Soldaini and Eugene Yang},
  eprint = {2404.08071},
  primaryclass = {cs.IR},
  title = {Overview of the TREC 2023 NeuCLIR Track},
  year = {2024},
}
""",
    )


class NeuCLIR2023RetrievalHardNegatives(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NeuCLIR2023RetrievalHardNegatives",
        description="The task involves identifying and retrieving the documents that are relevant to the queries. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://neuclir.github.io/",
        dataset={
            "path": "mteb/NeuCLIR2023RetrievalHardNegatives",
            "revision": "efb44bb789b9f82223aa819e18dba3748fd09e33",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_20",
        date=("2022-08-01", "2023-06-30"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="odc-by",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{lawrie2024overview,
  archiveprefix = {arXiv},
  author = {Dawn Lawrie and Sean MacAvaney and James Mayfield and Paul McNamee and Douglas W. Oard and Luca Soldaini and Eugene Yang},
  eprint = {2404.08071},
  primaryclass = {cs.IR},
  title = {Overview of the TREC 2023 NeuCLIR Track},
  year = {2024},
}
""",
        adapted_from=["NeuCLIR2022Retrieval"],
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "en": ["eng-Latn"],
    "de": ["deu-Latn"],
    "es": ["spa-Latn"],
    "fr": ["fra-Latn"],
    "hi": ["hin-Deva"],
    "th": ["tha-Thai"],
}


class MTOPIntentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MTOPIntentClassification",
        dataset={
            "path": "mteb/MTOPIntentClassification",
            "revision": "2992d820f31312593c49a4890430aadadb0f0039",
        },
        description="MTOP: Multilingual Task-Oriented Semantic Parsing",
        reference="https://arxiv.org/pdf/2008.09335.pdf",
        category="t2c",
        modalities=["text"],
        type="Classification",
        eval_splits=["validation", "test"],
        eval_langs=_LANGUAGES,
        main_score="accuracy",
        date=("2020-01-01", "2020-12-31"),
        domains=["Spoken", "Spoken"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{li-etal-2021-mtop,
  address = {Online},
  author = {Li, Haoran  and
Arora, Abhinav  and
Chen, Shuohui  and
Gupta, Anchit  and
Gupta, Sonal  and
Mehdad, Yashar},
  booktitle = {Proceedings of the 16th Conference of the European Chapter of the Association for Computational Linguistics: Main Volume},
  doi = {10.18653/v1/2021.eacl-main.257},
  editor = {Merlo, Paola  and
Tiedemann, Jorg  and
Tsarfaty, Reut},
  month = apr,
  pages = {2950--2962},
  publisher = {Association for Computational Linguistics},
  title = {{MTOP}: A Comprehensive Multilingual Task-Oriented Semantic Parsing Benchmark},
  url = {https://aclanthology.org/2021.eacl-main.257},
  year = {2021},
}
""",
        prompt="Classify the intent of the given utterance in task-oriented conversation",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VisRAGRetMPDocVQA(AbsTaskRetrieval):
    """VisRAG Retrieval task for MP-DocVQA industrial documents.

    The corpus contains scanned pages from multi-page industrial documents and
    the queries are questions targeting specific pages.  Each query has one
    relevant page image.
    """

    metadata = TaskMetadata(
        name="VisRAGRetMPDocVQA",
        description="Benchmark the ability to retrieve specific relevant pages from multi-page documents and generate answers based on visual evidence.",
        reference="https://arxiv.org/abs/2212.05935",
        type="Retrieval",
        task_subtypes=["Image Text Retrieval"],
        category="t2i",
        modalities=["text", "image"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_10",
        dataset={
            "path": "mteb/VisRAGRetMPDocVQA",
            "revision": "b1feea8bb4b37ade699d6974d71a11868866569f",
        },
        date=("1900-01-01", "2020-12-31"),
        domains=["Web", "Non-fiction"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{tito2023hierarchicalmultimodaltransformersmultipage,
  archiveprefix = {arXiv},
  author = {Rubèn Tito and Dimosthenis Karatzas and Ernest Valveny},
  eprint = {2212.05935},
  primaryclass = {cs.CV},
  title = {Hierarchical multimodal transformers for Multi-Page DocVQA},
  url = {https://arxiv.org/abs/2212.05935},
  year = {2023},
}""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VABBRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="VABBRetrieval",
        description="This dataset contains the fourteenth edition of the Flemish Academic Bibliography for the Social "
        "Sciences and Humanities (VABB-SHW), a database of academic publications from the social sciences "
        "and humanities authored by researchers affiliated to Flemish universities (more information). "
        "Publications in the database are used as one of the parameters of the Flemish performance-based "
        "research funding system",
        reference="https://zenodo.org/records/14214806",
        dataset={
            "path": "clips/mteb-nl-vabb-ret",
            "revision": "af4a1e5b3ed451103894f86ff6b3ce85085d7b48",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2009-11-01", "2010-01-01"),
        domains=["Academic", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@dataset{aspeslagh2024vabb,
  author = {Aspeslagh, Pieter and Guns, Raf and Engels, Tim C. E.},
  doi = {10.5281/zenodo.14214806},
  publisher = {Zenodo},
  title = {VABB-SHW: Dataset of Flemish Academic Bibliography for the Social Sciences and Humanities (edition 14)},
  url = {https://doi.org/10.5281/zenodo.14214806},
  year = {2024},
}
""",
        prompt={
            "query": "Gegeven een titel, haal de wetenschappelijke abstract op die het beste bij de titel past"
        },
    )



from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

zeroentropy_zembed_1 = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs={
        "trust_remote_code": True,
        "model_kwargs": {"torch_dtype": "bfloat16"},
    },
    name="zeroentropy/zembed-1",
    revision="fc4dc6ddd4a4f609978834de537b1f9f5cb29405",
    release_date="2025-09-16",
    languages=["eng-Latn"],
    n_parameters=4022468096,
    n_embedding_parameters=388956160,
    memory_usage_mb=7672,
    max_tokens=40960,
    embed_dim=2560,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["Sentence Transformers", "PyTorch"],
    reference="https://huggingface.co/zeroentropy/zembed-1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="Qwen/Qwen3-4B",
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation="""@misc{pipitone2025zeloeloinspiredtrainingmethod,
      title={zELO: ELO-inspired Training Method for Rerankers and Embedding Models},
      author={Nicholas Pipitone and Ghita Houir Alami and Advaith Avadhanam and Anton Kaminskyi and Ashley Khoo},
      year={2025},
      eprint={2509.12541},
      archivePrefix={arXiv},
      primaryClass={cs.AI},
      url={https://arxiv.org/abs/2509.12541},
}""",
    contacts=None,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class NQPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NQ-PL",
        description="Natural Questions: A Benchmark for Question Answering Research",
        reference="https://ai.google.com/research/NaturalQuestions/",
        dataset={
            "path": "mteb/NQ-PL",
            "revision": "b784c0399830a24e4ce0f9df2bb5b6be7fc8b246",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2023-05-01", "2024-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["NQ"],
    )


class NQPLHardNegatives(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NQ-PLHardNegatives",
        description="Natural Questions: A Benchmark for Question Answering Research. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://ai.google.com/research/NaturalQuestions/",
        dataset={
            "path": "mteb/NQ-PLHardNegatives",
            "revision": "cd0cab9433aa7f4e939883d822fb93e362fd493a",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),  # original publication year
        domains=["Written", "Encyclopaedic"],
        task_subtypes=["Question answering"],
        license="cc-by-nc-sa-3.0",  # original license
        annotations_creators="human-annotated",  # original annotation method
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["NQ"],
    )



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class XStance(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="XStance",
        dataset={
            "path": "mteb/XStance",
            "revision": "33c45c244e7c974f0c206372285a37e3f000f65a",
        },
        description="A Multilingual Multi-Target Dataset for Stance Detection in French, German, and Italian.",
        reference="https://github.com/ZurichNLP/xstance",
        category="t2t",
        modalities=["text"],
        type="PairClassification",
        eval_splits=["test"],
        eval_langs={
            "de": ["deu-Latn"],
            "fr": ["fra-Latn"],
            "it": ["ita-Latn"],
        },
        main_score="max_ap",
        date=("2011-01-01", "2020-12-31"),
        domains=["Social", "Written"],
        task_subtypes=["Political classification"],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{vamvas2020xstance,
  address = {Zurich, Switzerland},
  author = {Vamvas, Jannis and Sennrich, Rico},
  booktitle = {Proceedings of the 5th Swiss Text Analytics Conference (SwissText)  16th Conference on Natural Language Processing (KONVENS)},
  month = {jun},
  title = {{X-Stance}: A Multilingual Multi-Target Dataset for Stance Detection},
  url = {http://ceur-ws.org/Vol-2624/paper9.pdf},
  year = {2020},
}
""",
    )



"""EventLogger Main Class

Provides a concise API for logging various events
"""

import atexit
import logging
import os
from concurrent.futures import ThreadPoolExecutor
from typing import Any

from .models import (
    BaseEvent,
    BenchmarkChangeEvent,
    FilterChangeEvent,
    PageViewEvent,
    TableDownloadEvent,
    TableSwitchEvent,
)
from .storage import MongoDBStorage

logger = logging.getLogger(__name__)


class EventLogger:
    """Event logger main class

    Features:
    - Uses thread pool for asynchronous writes, non-blocking main thread
    - Silent failure, does not affect main business logic
    - Supports all predefined event types
    - Supports custom events

    Usage example:
        ```python
        from event_logger import EventLogger

        # Initialize (reads MONGO_URI from environment variable)
        event_logger = EventLogger()

        # Log filter change
        event_logger.log_filter_change(
            session_id="s_xyz789",
            filter_name="task_type",
            new_value="classification",
            benchmark="MTEB-English"
        )
        ```
    """

    def __init__(
        self,
        mongo_uri: str | None = None,
        database: str | None = None,
        collection: str | None = None,
        max_workers: int = 2,
    ):
        """Initialize event logger

        Args:
            mongo_uri: MongoDB connection string, defaults to reading from environment variable MONGO_URI
            database: Database name
            collection: Collection name
            max_workers: Maximum number of worker threads in thread pool
        """
        self._storage: MongoDBStorage | None = None
        self._storage_kwargs = {
            "uri": mongo_uri,
            "database": database,
            "collection": collection,
        }

        # If MONGO_URI is not set, disable event logging and log one WARNING (do not create MongoDBStorage)
        effective_uri = mongo_uri or os.getenv("MONGO_URI")
        self._storage_disabled = not (effective_uri and effective_uri.strip())
        if self._storage_disabled:
            logger.warning(
                "MONGO_URI not configured; event logging disabled. "
                "Set environment variable MONGO_URI or pass mongo_uri to enable."
            )

        # Thread pool for asynchronous writes
        self._executor = ThreadPoolExecutor(max_workers=max_workers)

        # Register cleanup on exit
        atexit.register(self._cleanup)

        self._initialized = False

    def _ensure_storage(self) -> bool:
        """Ensure storage layer is initialized

        Returns:
            Whether initialization was successful
        """
        if self._storage is not None:
            return True
        if self._storage_disabled:
            return False

        try:
            self._storage = MongoDBStorage(**self._storage_kwargs)
            self._initialized = True
            return True
        except Exception as e:
            logger.warning(
                f"EventLogger initialization failed (will silently skip event logging): {e}"
            )
            return False

    def _log_async(self, event: BaseEvent):
        """Asynchronously write event (executed in background thread)

        Args:
            event: Event object
        """
        if not self._ensure_storage():
            return

        try:
            event_data = event.to_mongo_dict()
            self._storage.insert(event_data)
        except Exception as e:
            # Silent failure
            logger.debug(f"Event logging failed: {e}")

    def log(self, event: BaseEvent):
        """Log event (asynchronous, non-blocking)

        Args:
            event: Event object
        """
        self._executor.submit(self._log_async, event)

    def log_raw(
        self,
        event_name: str,
        session_id: str,
        benchmark: str | None = None,
        filters: dict[str, Any] | None = None,
        properties: dict[str, Any] | None = None,
        **kwargs,
    ):
        """Log raw event (generic method)

        Args:
            event_name: Event name
            session_id: Session ID
            benchmark: Benchmark name
            filters: Filter conditions snapshot
            properties: Event properties
            **kwargs: Other custom fields
        """
        event = BaseEvent(
            event_name=event_name,
            session_id=session_id,
            benchmark=benchmark,
            filters=filters,
            properties=properties,
            **kwargs,
        )
        self.log(event)

    # ============ Convenience Methods ============

    def log_page_view(self, session_id: str, benchmark: str | None = None, **kwargs):
        """Log page view event

        Args:
            session_id: Session ID
            benchmark: Current benchmark
            **kwargs (Any): Additional context.
        """
        event = PageViewEvent(session_id=session_id, benchmark=benchmark, **kwargs)
        self.log(event)

    def log_benchmark_change(
        self, session_id: str, new_value: str, old_value: str | None = None, **kwargs
    ):
        """Log benchmark change event

        Args:
            session_id: Session ID
            new_value: Newly selected benchmark
            old_value: Previous benchmark
            **kwargs (Any): Additional context.
        """
        event = BenchmarkChangeEvent.create(
            session_id=session_id, old_value=old_value, new_value=new_value, **kwargs
        )
        self.log(event)

    def log_filter_change(
        self,
        session_id: str,
        filter_name: str,
        new_value: Any,
        old_value: Any = None,
        benchmark: str | None = None,
        filters: dict[str, Any] | None = None,
        **kwargs,
    ):
        """Log filter change event

        Args:
            session_id: Session ID
            filter_name: Filter name (e.g., task_type, domain, language, etc.)
            new_value: New value
            old_value: Old value
            benchmark: Current benchmark
            filters: Snapshot of all current filter conditions
            **kwargs (Any): Additional context.
        """
        event = FilterChangeEvent.create(
            session_id=session_id,
            filter_name=filter_name,
            new_value=new_value,
            old_value=old_value,
            benchmark=benchmark,
            filters=filters,
            **kwargs,
        )
        self.log(event)

    def log_table_switch(
        self,
        session_id: str,
        new_table: str,
        old_table: str | None = None,
        benchmark: str | None = None,
        **kwargs,
    ):
        """Log table switch event

        Args:
            session_id: Session ID
            new_table: New table
            old_table: Old table
            benchmark: Current benchmark
            **kwargs (Any): Additional context.
        """
        event = TableSwitchEvent.create(
            session_id=session_id,
            old_table=old_table,
            new_table=new_table,
            benchmark=benchmark,
            **kwargs,
        )
        self.log(event)

    def log_table_download(
        self,
        session_id: str,
        benchmark: str | None = None,
        format: str = "csv",
        row_count: int | None = None,
        **kwargs,
    ):
        """Log table download event

        Args:
            session_id: Session ID
            benchmark: Current benchmark
            format: Download format (e.g., csv, json)
            row_count: Number of rows downloaded
            **kwargs (Any): Additional context.
        """
        event = TableDownloadEvent.create(
            session_id=session_id,
            benchmark=benchmark,
            format=format,
            row_count=row_count,
            **kwargs,
        )
        self.log(event)

    def _cleanup(self):
        """Clean up resources"""
        self._executor.shutdown(wait=True)
        if self._storage:
            self._storage.close()



"""Classes for `eval.yaml`.

Limited version of eval.yaml model based on https://huggingface.co/datasets/Idavidrein/gpqa/blob/main/eval.yaml
"""

from __future__ import annotations

from typing import Literal

import yaml
from pydantic import BaseModel, ConfigDict, Field


class HFEvalTaskConfig(BaseModel):
    id: str
    config: str | None = Field(
        None,
        description="HF dataset config name (typically languages), e.g. `eng`, `fra`. None for mean all configs are used. Config is also sometimes referred to as subset or in mteb as `hf_subset`.",
    )
    split: str | None = Field(
        None,
        description="HF dataset split name, e.g. `test` or `train`. None for mean all splits are used.",
    )


class HFEvalMeta(BaseModel):
    model_config = ConfigDict(extra="forbid")

    # TODO:
    #  prompts? (what to do with default?)
    #  default metric
    #  more metadata about task? (domains etc.)
    name: str
    description: str
    evaluation_framework: Literal["mteb"] = "mteb"
    tasks: list[HFEvalTaskConfig]

    def merge(self, other: HFEvalMeta) -> HFEvalMeta:
        existing_tasks = {t.id for t in self.tasks}
        for other_task in other.tasks:
            if other_task.id not in existing_tasks:
                self.tasks.append(other_task)
        self.tasks = sorted(self.tasks, key=lambda t: t.id)
        return self

    def to_yaml(self) -> str:
        task_config_dict = self.model_dump(exclude_none=True)
        return yaml.safe_dump(
            task_config_dict,
            # to keep order id, split, config
            sort_keys=False,
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SlovakHateSpeechClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakHateSpeechClassification",
        description="The dataset contains posts from a social network with human annotations for hateful or offensive language in Slovak.",
        reference="https://huggingface.co/datasets/TUKE-KEMT/hate_speech_slovak",
        dataset={
            "path": "TUKE-KEMT/hate_speech_slovak",
            "revision": "f9301b9937128c9c0b636fa6da203aeb046479f4",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2024-05-25", "2024-06-06"),
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        superseded_by="SlovakHateSpeechClassification.v2",
    )


class SlovakHateSpeechClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakHateSpeechClassification.v2",
        description="The dataset contains posts from a social network with human annotations for hateful or offensive language in Slovak. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://huggingface.co/datasets/TUKE-KEMT/hate_speech_slovak",
        dataset={
            "path": "mteb/slovak_hate_speech",
            "revision": "691fe861df0ffa25066cbf6da8e64ebd296af6ab",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2024-05-25", "2024-06-06"),
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        adapted_from=["SlovakHateSpeechClassification"],
    )



from .argu_ana_pl_retrieval import ArguAnaPL
from .cqadupstack_pl_retrieval import (
    CQADupstackAndroidRetrievalPL,
    CQADupstackEnglishRetrievalPL,
    CQADupstackGamingRetrievalPL,
    CQADupstackGisRetrievalPL,
    CQADupstackMathematicaRetrievalPL,
    CQADupstackPhysicsRetrievalPL,
    CQADupstackProgrammersRetrievalPL,
    CQADupstackStatsRetrievalPL,
    CQADupstackTexRetrievalPL,
    CQADupstackUnixRetrievalPL,
    CQADupstackWebmastersRetrievalPL,
    CQADupstackWordpressRetrievalPL,
)
from .db_pedia_pl_retrieval import DBPediaPL, DBPediaPLHardNegatives
from .fi_qapl_retrieval import FiQAPLRetrieval
from .hotpot_qapl_retrieval import HotpotQAPL, HotpotQAPLHardNegatives
from .msmarcopl_retrieval import MSMARCOPL, MSMARCOPLHardNegatives
from .nf_corpus_pl_retrieval import NFCorpusPL
from .nqpl_retrieval import NQPL, NQPLHardNegatives
from .pugg_retrieval import PUGGRetrieval
from .quora_pl_retrieval import QuoraPLRetrieval, QuoraPLRetrievalHardNegatives
from .sci_fact_pl_retrieval import SciFactPL
from .scidocspl_retrieval import SCIDOCSPL
from .touche2020_pl_retrieval import Touche2020PL
from .treccovidpl_retrieval import TRECCOVIDPL

__all__ = [
    "MSMARCOPL",
    "NQPL",
    "SCIDOCSPL",
    "TRECCOVIDPL",
    "ArguAnaPL",
    "CQADupstackAndroidRetrievalPL",
    "CQADupstackEnglishRetrievalPL",
    "CQADupstackGamingRetrievalPL",
    "CQADupstackGisRetrievalPL",
    "CQADupstackMathematicaRetrievalPL",
    "CQADupstackPhysicsRetrievalPL",
    "CQADupstackProgrammersRetrievalPL",
    "CQADupstackStatsRetrievalPL",
    "CQADupstackTexRetrievalPL",
    "CQADupstackUnixRetrievalPL",
    "CQADupstackWebmastersRetrievalPL",
    "CQADupstackWordpressRetrievalPL",
    "DBPediaPL",
    "DBPediaPLHardNegatives",
    "FiQAPLRetrieval",
    "HotpotQAPL",
    "HotpotQAPLHardNegatives",
    "MSMARCOPLHardNegatives",
    "NFCorpusPL",
    "NQPLHardNegatives",
    "PUGGRetrieval",
    "QuoraPLRetrieval",
    "QuoraPLRetrievalHardNegatives",
    "SciFactPL",
    "Touche2020PL",
]
